@@ -1,9 +1,10 @@
 from __future__ import annotations
 
+from copy import deepcopy
 from functools import partial
-from typing import Any, Callable, ClassVar, Collection, Iterable, Sequence, cast
+from typing import Any, Callable, ClassVar, cast
+from collections.abc import Collection, Iterable, Sequence
 
-import django
 from django.core.exceptions import ImproperlyConfigured
 from django.db.models import (
     Field,
@@ -14,14 +15,15 @@ from django.db.models import (
     OneToOneField,
     options,
 )
+from django.db.models import UniqueConstraint
 from django.db.models.base import ModelBase
 from django.db.models.signals import post_init
 from django.utils.functional import cached_property
 
+from modeltranslation.utils import get_translation_fields
 from modeltranslation import settings as mt_settings
 from modeltranslation.fields import (
     NONE,
-    LanguageCacheSingleObjectDescriptor,
     TranslatedManyToManyDescriptor,
     TranslatedRelationIdDescriptor,
     TranslationFieldDescriptor,
@@ -31,14 +33,31 @@ from modeltranslation.fields import (
 from modeltranslation.manager import (
     MultilingualManager,
     MultilingualQuerysetManager,
-    append_translated,
     rewrite_lookup_key,
 )
 from modeltranslation.thread_context import auto_populate_mode
-from modeltranslation.utils import build_localized_fieldname, parse_field
+from modeltranslation.utils import (
+    build_localized_fieldname,
+    parse_field,
+    get_language,
+)
 
+# Re-export the decorator for convenience
+from modeltranslation.decorators import register
+
+from ._compat import is_hidden, build_refresh_from_db
 from ._typing import _ListOrTuple
 
+__all__ = [
+    "AlreadyRegistered",
+    "DescendantRegistered",
+    "NotRegistered",
+    "TranslationOptions",
+    "Translator",
+    "register",
+    "translator",
+]
+
 
 class AlreadyRegistered(Exception):
     pass
@@ -57,7 +76,7 @@ class FieldsAggregationMetaClass(type):
     Metaclass to handle custom inheritance of fields between classes.
     """
 
-    fields: Iterable[str]
+    fields: Sequence[str]
 
     def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> type:
         attrs["fields"] = set(attrs.get("fields", ()))
@@ -76,11 +95,11 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
     attributes.
 
     Options instances hold info about translatable fields for a model and its
-    superclasses. The ``local_fields`` and ``fields`` attributes are mappings
+    superclasses. The ``local_fields`` and ``all_fields`` attributes are mappings
     from fields to sets of their translation fields; ``local_fields`` contains
     only those fields that are handled in the model's database table (those
     inherited from abstract superclasses, unless there is a concrete superclass
-    in between in the inheritance chain), while ``fields`` also includes fields
+    in between in the inheritance chain), while ``all_fields`` also includes fields
     inherited from concrete supermodels (giving all translated fields available
     on a model).
 
@@ -89,7 +108,9 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
     ``related_fields`` contains names of reverse lookup fields.
     """
 
-    required_languages: ClassVar[_ListOrTuple[str] | dict[str, _ListOrTuple[str]]] = ()
+    required_languages: ClassVar[_ListOrTuple[str] | dict[str, _ListOrTuple[str]]] = (
+        mt_settings.REQUIRED_LANGUAGES
+    )
 
     def __init__(self, model: Model) -> None:
         """
@@ -99,7 +120,7 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
         self.registered = False
         self.related = False
         self.local_fields: dict[str, set[TranslationField]] = {f: set() for f in self.fields}
-        self.fields: dict[str, set[TranslationField]] = {f: set() for f in self.fields}
+        self.all_fields: dict[str, set[TranslationField]] = {f: set() for f in self.fields}
         self.related_fields: list[str] = []
 
     def validate(self) -> None:
@@ -108,13 +129,13 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
         """
         # TODO: at the moment only required_languages is validated.
         # Maybe check other options as well?
-        if self.required_languages:
-            if isinstance(self.required_languages, (tuple, list)):
-                self._check_languages(self.required_languages)
+        if value := self.required_languages:
+            if isinstance(value, (tuple, list)):
+                self._check_languages(value)
             else:
-                self._check_languages(self.required_languages.keys(), extra=("default",))
-                for fieldnames in self.required_languages.values():
-                    if any(f not in self.fields for f in fieldnames):
+                self._check_languages(value.keys(), extra=("default",))
+                for fieldnames in value.values():
+                    if any(f not in self.all_fields for f in fieldnames):
                         raise ImproperlyConfigured(
                             "Fieldname in required_languages which is not in fields option."
                         )
@@ -136,24 +157,24 @@ class TranslationOptions(metaclass=FieldsAggregationMetaClass):
         """
         if other.model._meta.abstract:
             self.local_fields.update(other.local_fields)
-        self.fields.update(other.fields)
+        self.all_fields.update(other.all_fields)
 
     def add_translation_field(self, field: str, translation_field):
         """
         Add a new translation field to both fields dicts.
         """
         self.local_fields[field].add(translation_field)
-        self.fields[field].add(translation_field)
+        self.all_fields[field].add(translation_field)
 
     def get_field_names(self) -> list[str]:
         """
         Return name of all fields that can be used in filtering.
         """
-        return list(self.fields.keys()) + self.related_fields
+        return list(self.all_fields.keys()) + self.related_fields
 
     def __str__(self) -> str:
         local = tuple(self.local_fields.keys())
-        inherited = tuple(set(self.fields.keys()) - set(local))
+        inherited = tuple(set(self.all_fields.keys()) - set(local))
         return "%s: %s + %s" % (self.__class__.__name__, local, inherited)
 
 
@@ -299,6 +320,33 @@ def patch_constructor(model: type[Model]) -> None:
     model.__init__ = new_init
 
 
+def patch_constraints(model: type[Model], opts: TranslationOptions) -> None:
+    def add_unique_together():
+        for constraint in model._meta.unique_together:
+            for field_name in opts.fields:
+                if field_name in constraint:
+                    new_constraint = list(constraint)
+                    for translated_name in get_translation_fields(field_name):
+                        new_constraint[constraint.index(field_name)] = translated_name
+                        yield new_constraint
+
+    def add_constraints():
+        for c in model._meta.constraints:
+            if isinstance(c, UniqueConstraint):
+                for field_name in opts.fields:
+                    if field_name in c.fields:
+                        for translated_name in get_translation_fields(field_name):
+                            new_constraint = deepcopy(c)
+                            new_fields = list(new_constraint.fields)
+                            new_fields[new_fields.index(field_name)] = translated_name
+                            new_constraint.name += f"-{translated_name}"
+                            new_constraint.fields = new_fields
+                            yield new_constraint
+
+    model._meta.unique_together += tuple(add_unique_together())  # type: ignore[operator]
+    model._meta.constraints += tuple(add_constraints())
+
+
 def delete_mt_init(sender: type[Model], instance: Model, **kwargs: Any) -> None:
     if hasattr(instance, "_mt_init"):
         del instance._mt_init
@@ -355,16 +403,8 @@ def patch_refresh_from_db(model: type[Model]) -> None:
     """
     if not hasattr(model, "refresh_from_db"):
         return
-    old_refresh_from_db = model.refresh_from_db
 
-    def new_refresh_from_db(
-        self, using: str | None = None, fields: Sequence[str] | None = None
-    ) -> None:
-        if fields is not None:
-            fields = append_translated(self.__class__, fields)  # type: ignore[assignment]
-        return old_refresh_from_db(self, using, fields)
-
-    model.refresh_from_db = new_refresh_from_db
+    model.refresh_from_db = build_refresh_from_db(model.refresh_from_db)
 
 
 def delete_cache_fields(model: type[Model]) -> None:
@@ -421,10 +461,10 @@ def populate_translation_fields(sender: type[Model], kwargs: Any):
 
     opts = translator.get_options_for_model(sender)
     for key, val in list(kwargs.items()):
-        if key in opts.fields:
+        if key in opts.all_fields:
             if populate == "all":
                 # Set the value for every language.
-                for translation_field in opts.fields[key]:
+                for translation_field in opts.all_fields[key]:
                     kwargs.setdefault(translation_field.name, val)
             elif populate == "default":
                 default = build_localized_fieldname(key, mt_settings.DEFAULT_LANGUAGE)
@@ -443,17 +483,21 @@ def patch_related_object_descriptor_caching(ro_descriptor):
     language-aware caching.
     """
 
-    class NewSingleObjectDescriptor(LanguageCacheSingleObjectDescriptor, ro_descriptor.__class__):
-        pass
+    class NewRelated(ro_descriptor.related.__class__):
+        def get_cache_name(self) -> str:
+            """
+            Used in django > 2.x
+            """
+            return self.cache_name
 
-    if django.VERSION[0] >= 2:
-        ro_descriptor.related.get_cache_name = partial(
-            NewSingleObjectDescriptor.get_cache_name,
-            ro_descriptor,
-        )
+        @property
+        def cache_name(self):
+            """
+            Used in django >= 5.1
+            """
+            return build_localized_fieldname(self.get_accessor_name(), get_language())
 
-    ro_descriptor.accessor = ro_descriptor.related.get_accessor_name()
-    ro_descriptor.__class__ = NewSingleObjectDescriptor
+    ro_descriptor.related.__class__ = NewRelated
 
 
 class Translator:
@@ -547,6 +591,9 @@ class Translator:
         # Patch __init__ to rewrite fields
         patch_constructor(model)
 
+        # Patch constraints to correctly handle new fields
+        patch_constraints(model, opts)
+
         # Connect signal for model
         post_init.connect(delete_mt_init, sender=model)
 
@@ -585,7 +632,7 @@ class Translator:
                 setattr(model, field.get_attname(), desc)
 
                 # Set related field names on other model
-                if not field.remote_field.is_hidden():
+                if not is_hidden(field.remote_field):
                     other_opts = self._get_options_for_model(field.remote_field.model)
                     other_opts.related = True
                     other_opts.related_fields.append(field.related_query_name())
@@ -596,7 +643,7 @@ class Translator:
                 # Fix translated_field caching for SingleRelatedObjectDescriptor
                 sro_descriptor = getattr(
                     field.remote_field.model,
-                    field.remote_field.get_accessor_name(),  # type: ignore[arg-type]
+                    field.remote_field.get_accessor_name(),
                 )
                 patch_related_object_descriptor_caching(sro_descriptor)
 
@@ -676,9 +723,7 @@ class Translator:
         """
         opts = self._get_options_for_model(model)
         if not opts.registered and not opts.related:
-            raise NotRegistered(
-                'The model "%s" is not registered for ' "translation" % model.__name__
-            )
+            raise NotRegistered('The model "%s" is not registered for translation' % model.__name__)
         return opts
 
     def execute_lazy_operations(self) -> None:
@@ -691,7 +736,3 @@ class Translator:
 
 # This global object represents the singleton translator object
 translator = Translator()
-
-
-# Re-export the decorator for convenience
-from modeltranslation.decorators import register  # NOQA re-export
